//+------------------------------------------------------------------+
//|                                              TradeCopyTester.mq4 |
//|                                    Copyright  2008, fxtrader625 |
//|                                  http://www.forexampletrades.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, fxtrader625"
#property link      "http://www.forexampletrades.com"

#import "TradeCopier.ex4"
   void CopyTrade(int ticket, bool reverse);
   void UpdateTrade(int ticket, bool reverse);
   void CloseTrade(int ticket, double volume = -1);
   void DeleteTrade(int ticket);
#import

int testcounter = 0;
datetime testtime = 0;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
   testcounter = 0;
   testtime = CurTime();
   // Close and delete all trades
   for (int i = OrdersTotal()-1; i >= 0; i--)
   {
      Sleep(1000);
      if (OrderSelect(i, SELECT_BY_POS))
      {
         int ticket = OrderTicket();
         if (OrderType() == OP_BUY)
         {
            OrderClose(ticket, OrderLots(), Bid, 3);
            CloseTrade(ticket, -1);
         }
         else if (OrderType() == OP_SELL)
         {
            OrderClose(ticket, OrderLots(), Ask, 3);
            CloseTrade(ticket, -1);
         }
         else
         {
            OrderDelete(ticket);
            DeleteTrade(ticket);
         }
      }
   }
   return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   if (CurTime() < testtime + 60)
   {
      return(0);
   }
   testtime = CurTime();
   Comment("Test time loop ", testcounter);
   switch (testcounter)
   {
   case 0:
      // Test copy trade 1
      int ticket = OrderSend(Symbol(), OP_BUY, 0.2, Ask, 3, Ask - 50 * Point, Ask + 50 * Point, NULL, 0);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 1:
      // Test copy reversed trade 1
      ticket = OrderSend(Symbol(), OP_BUY, 0.4, Ask, 3, Ask - 30 * Point, Ask + 100 * Point, NULL, 1);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   case 2:
      // Test copy trade 2
      ticket = OrderSend(Symbol(), OP_SELL, 0.6, Bid, 3, Bid + 50 * Point, Bid - 50 * Point, NULL, 2);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 3:
      // Test copy reversed trade 2
      ticket = OrderSend(Symbol(), OP_SELL, 0.8, Bid, 3, Bid + 30 * Point, Bid - 100 * Point, NULL, 3);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   case 4:
      // Test copy trade 3   
      ticket = OrderSend(Symbol(), OP_BUYSTOP, 1.0, Ask + 50 * Point, 3, Ask - 30 * Point, Ask + 130 * Point, NULL, 4);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 5:
      // Test copy reversed trade 3
      ticket = OrderSend(Symbol(), OP_BUYSTOP, 1.2, Ask + 50 * Point, 3, Ask - 30 * Point, Ask + 130 * Point, NULL, 5);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   case 6:
      // Test copy trade 4
      ticket = OrderSend(Symbol(), OP_SELLSTOP, 1.4, Bid - 50 * Point, 3, Bid + 50 * Point, Bid - 150 * Point, NULL, 6);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 7:
      // Test copy reversed trade 4
      ticket = OrderSend(Symbol(), OP_SELLSTOP, 1.6, Bid - 50 * Point, 3, Bid + 50 * Point, Bid - 150 * Point, NULL, 7);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   case 8:
      // Test copy trade 5
      ticket = OrderSend(Symbol(), OP_BUYLIMIT, 1.8, Ask - 50 * Point, 3, Ask - 80 * Point, Ask + 80 * Point, NULL, 8);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 9:
      // Test copy reversed trade 5
      ticket = OrderSend(Symbol(), OP_BUYLIMIT, 2.0, Ask - 50 * Point, 3, Ask - 80 * Point, Ask + 80 * Point, NULL, 9);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   case 10:
      // Test copy trade 6
      ticket = OrderSend(Symbol(), OP_SELLLIMIT, 2.2, Bid + 50 * Point, 3, Bid + 100 * Point, Bid - 50 * Point, NULL, 10);
      if (ticket != -1)
      {
         CopyTrade(ticket, false);
      }
      break;
   case 11:
      // Test copy reversed trade 6
      ticket = OrderSend(Symbol(), OP_SELLLIMIT, 2.4, Bid + 50 * Point, 3, Bid + 100 * Point, Bid - 50 * Point, NULL, 11);
      if (ticket != -1)
      {
         CopyTrade(ticket, true);
      }
      break;
   default:
      if (testcounter >= 12 && testcounter < 24)
      {
         // Test modify trade 1
         ticket = GetTicketByMagicNumber(testcounter - 12);
         if (OrderSelect(ticket, SELECT_BY_TICKET))
         {
            if (OrderType() < 2)
            {
               OrderModify(ticket, 0, OrderStopLoss() - 5 * Point, OrderTakeProfit() + 5 * Point, 0);
            }
            else
            {
               OrderModify(ticket, OrderOpenPrice() + Point, OrderStopLoss() - 5 * Point, OrderTakeProfit() + 5 * Point, 0);
            }
            if (testcounter % 2 == 0)
            {
               UpdateTrade(ticket, false);
            }
            else
            {
               UpdateTrade(ticket, true);
            }
         }
      }
      break;
   }
   testcounter++;
   return(0);
}

int GetTicketByMagicNumber(int magic)
{
   for (int i = 0; i < OrdersTotal(); i++)
   {
      OrderSelect(i, SELECT_BY_POS);
      if (OrderMagicNumber() == magic)
      {
         return (OrderTicket());
      }
   }
   return (0);
}

